/**
 * Rekursive Palindrom- und Anagramm-Erkennung.
 * 
 * @author Owyn
 * @version 2008-05-31
 */
public class Words {
    
    /**
     * Testet, ob ein &uuml;bergebener String ein Palindrom ist, also von vorne
     * und hinten gelesen gleich lautet.
     * <br>Gro&szlig;- und Kleinschreibung wird ignoriert.
     * 
     * @param txt Zu &uuml;berpr&uuml;fendes Wort
     * @return true, wenn txt ein Palindrom ist, sonst false
     */
    public static boolean palindrom(String txt) {
        // Leerzeichen an Anfang und Ende entfernen und in Grobuchstaben umwandeln
        txt = txt.trim().toUpperCase();
        
        // Nur noch ein Buchstabe? Palindrom!
        if (txt.length() < 2) return true;
        
        // Erster und letzter Buchstabe unterscheiden sich? Kein Palindrom!
        if (txt.charAt(0) != txt.charAt(txt.length() - 1)) return false;
        
        // Ersten und letzten Buchstaben abschneiden, weiter testen
        return palindrom(txt.substring(1, txt.length() - 1));
    }
    
    /**
     * Testet, ob zwei &uuml;bergebene Strings Anagramme sind, also nur
     * durch Umstellen von Buchstaben ineinander umgewandelt werden k&ouml;nnen.
     * <br>Leerzeichen sowie Gro&szlig;- und Kleinschreibung werden ignoriert.
     * 
     * @param t1 Erster String
     * @param t2 Zweiter String
     * @return true, wenn t2 ein Anagramm von t1 ist, sonst false
     */
    public static boolean anagramm(String t1, String t2) {
        // Leerzeichen an Anfang und Ende entfernen und in Grobuchstaben umwandeln
        t1 = t1.trim().toUpperCase();
        t2 = t2.trim().toUpperCase();
        
        //System.out.println(txt1 + " - " + txt2); // Test-Ausgabe
        
        // t1 ist leer? Anagramm, wenn t2 auch leer, sonst keins!
        if (t1.length() == 0) return (t2.length() == 0);
        
        // Erstes Zeichen von t1 in t2 suchen
        int found = t2.indexOf(t1.charAt(0));
        
        // Nicht gefunden? Kein Anagramm!
        if (found == -1) return false;
        
        // Gefunden und nur noch ein Buchstabe?
        // Anagramm, wenn t2 auch nur einen Buchstaben lang ist, sonst keins!
        if (t1.length() < 2) return (t2.length() < 2);
        
        // Ersten Buchstaben von t1 aus t2 entfernen
        String t2new = t2.substring(0, found) + t2.substring(found + 1);
        
        // Anagramm suchen ohne ersten Buchstaben von t1
        return anagramm(t1.substring(1), t2new);
    }
    
}
